import path from "path";
import fs from "fs";
import { app, BrowserWindow, Menu, screen, dialog, ipcMain } from "electron";
import installExtension, { VUEJS3_DEVTOOLS } from "electron-devtools-installer";

import { exec } from "child_process";
import process from "process";
import IniParser from "ini-parser";
import logger from "./logger";

const isDev = process.env.NODE_ENV === "development";
let win: BrowserWindow | null = null;
let loadWin: any = null;

function createWin() {
  const { width, height } = screen.getPrimaryDisplay().bounds;
  // Menu.setApplicationMenu(null);
  // 创建浏览器窗口
  win = new BrowserWindow({
    // width: 860,
    // height: 520,
    width: width,
    height: height,
    frame: false,
    resizable: true,
    transparent: true,
    webPreferences: {
      preload: path.resolve(__dirname, "preload.js"),
      contextIsolation: true,
      webSecurity: false,
      sandbox: false,
    },
  });

  if (isDev) {
    win.loadURL(`http://localhost:8090`);
  } else {
    win.loadFile(path.resolve(__dirname, "index.html"));
  }

  win.webContents.once("dom-ready", () => {
    win?.show();
  });
  win.webContents.on("before-input-event", (event: any, input: any) => {
    if (input.key === "F12") {
      win?.webContents.openDevTools();
    }
  });
  // loadWin.destroy();

  win.on("closed", () => {
    win = null;
  });
}

function registEvent() {
  ipcMain.on("change-win-size", (event, args: string) => {
    // const { width, height } = screen.getPrimaryDisplay().workAreaSize;
    if (!win) return;
    const windowBounds = win.getBounds();
    const { width, height } =
      screen.getDisplayMatching(windowBounds).workAreaSize;

    let w = args === "big" ? width : 840;
    let h = args === "big" ? height : 500;
    if (args === "small") {
      win.setMinimumSize(w, h);
    }
    win.setSize(w, h);
    win.center();
  });
  ipcMain.on("window-min", () => {
    win?.minimize();
  });

  ipcMain.on("hide-app", () => {
    win?.hide();
  });

  ipcMain.on("get-page-size-ini", (event: any) => {
    const appPath = isDev ? process.cwd() : path.dirname(app.getPath("exe"));
    const iniFilePath = path.resolve(appPath, "page_size.ini");
    fs.readFile(iniFilePath, "utf8", (err: any, data: any) => {
      if (err) dialog.showErrorBox("tip", `page_size.ini文件不存在！`);
      const content = IniParser.parse(data);
      event.sender.send("got-page-size-ini", content);
    });
  });

  ipcMain.handle("get-app-ini", async () => {
    const appPath = isDev ? process.cwd() : path.dirname(app.getPath("exe"));
    const iniFilePath = path.resolve(appPath, "app.ini");
    try {
      const data = await fs.promises.readFile(iniFilePath, "utf8");
      const content = IniParser.parse(data);
      return content;
    } catch (err: any) {
      logger.error(`读取或解析 app.ini 文件失败: ${err.message}`);
      dialog.showErrorBox(
        "错误",
        `读取或解析 app.ini 文件失败！\n${err.message}`
      );
      return null; // 返回 null 表示失败
    }
  });
}

app.on("ready", async () => {
  if (isDev) {
    try {
      await installExtension(VUEJS3_DEVTOOLS);
    } catch (e: any) {
      console.error("Vue Devtools failed to install:", e.toString());
    }
  }

  logger.info(`app ready`);
  registEvent();

  createWin();
});
